@@ -22,6 +22,24 @@
 
 /* See Makefile for compilation details. */
 
+/*
+   Copyright (C) 1999-2009 Free Software Foundation, Inc.
+
+   This file is part of GNU Bash.
+   Bash is free software: you can redistribute it and/or modify
+   it under the terms of the GNU General Public License as published by
+   the Free Software Foundation, either version 3 of the License, or
+   (at your option) any later version.
+
+   Bash is distributed in the hope that it will be useful,
+   but WITHOUT ANY WARRANTY; without even the implied warranty of
+   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
+   GNU General Public License for more details.
+
+   You should have received a copy of the GNU General Public License
+   along with Bash.  If not, see <http://www.gnu.org/licenses/>.
+*/
+
 #include <config.h>
 
 #include <sys/types.h>
@@ -45,6 +63,7 @@
 #include "stdc.h"
 #include "bashgetopt.h"
 #include "maxpath.h"
+#include "common.h"
 
 #if !defined (errno)
 extern int errno;
@@ -79,6 +98,7 @@ extern char *strerror ();
 
 static int validate_path ();
 
+int
 pathchk_builtin (list)
      WORD_LIST *list;
 {
@@ -112,6 +132,8 @@ pathchk_builtin (list)
 }
 
 char *pathchk_doc[] = {
+	"Check pathnames for validity.",
+	"",
 	"Check each pathname argument for validity (i.e., it may be used to",
 	"create or access a file without casuing syntax errors) and portability",
 	"(i.e., no filename truncation will result).  If the `-p' option is",
@@ -302,7 +324,7 @@ validate_path (path, portability)
       if (!last_elem)
 	{
 	  exists = dir_ok (path);
-	  if (dir_ok == 0)
+	  if (exists == 0)
 	    {
 	      free (parent);
 	      return 1;
@@ -349,8 +371,8 @@ validate_path (path, portability)
   free (parent);
   if (strlen (path) > path_max)
     {
-      builtin_error ("path `%s' has length %d; exceeds limit of %d",
-	     path, strlen (path), path_max);
+      builtin_error ("path `%s' has length %lu; exceeds limit of %d",
+	     path, (unsigned long)strlen (path), path_max);
       return 1;
     }
 
