@@ -2,15 +2,35 @@
 
 /* See Makefile for compilation details. */
 
+/*
+   Copyright (C) 1999-2009 Free Software Foundation, Inc.
+
+   This file is part of GNU Bash.
+   Bash is free software: you can redistribute it and/or modify
+   it under the terms of the GNU General Public License as published by
+   the Free Software Foundation, either version 3 of the License, or
+   (at your option) any later version.
+
+   Bash is distributed in the hope that it will be useful,
+   but WITHOUT ANY WARRANTY; without even the implied warranty of
+   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
+   GNU General Public License for more details.
+
+   You should have received a copy of the GNU General Public License
+   along with Bash.  If not, see <http://www.gnu.org/licenses/>.
+*/
+
 #include "config.h"
 
 #include <stdio.h>
 #include "builtins.h"
 #include "shell.h"
 #include "bashgetopt.h"
+#include "common.h"
 
 extern char *ttyname ();
 
+int
 tty_builtin (list)
      WORD_LIST *list;
 {
@@ -40,6 +60,8 @@ tty_builtin (list)
 }
 
 char *tty_doc[] = {
+	"Display terminal name.",
+	"",
 	"tty writes the name of the terminal that is opened for standard",
 	"input to standard output.  If the `-s' option is supplied, nothing",
 	"is written; the exit status determines whether or not the standard",
