@@ -1,6 +1,6 @@
 /* histexpand.c -- history expansion. */
 
-/* Copyright (C) 1989-2009 Free Software Foundation, Inc.
+/* Copyright (C) 1989-2015 Free Software Foundation, Inc.
 
    This file contains the GNU History Library (History), a set of
    routines for managing the text of previously typed lines.
@@ -44,12 +44,14 @@
 
 #include "history.h"
 #include "histlib.h"
+#include "chardefs.h"
 
 #include "rlshell.h"
 #include "xmalloc.h"
 
 #define HISTORY_WORD_DELIMITERS		" \t\n;&()|<>"
 #define HISTORY_QUOTE_CHARACTERS	"\"'`"
+#define HISTORY_EVENT_DELIMITERS	"^$*%-"
 
 #define slashify_in_quotes "\\`\"$"
 
@@ -62,6 +64,10 @@ static char *subst_rhs;
 static int subst_lhs_len;
 static int subst_rhs_len;
 
+/* Characters that delimit history event specifications and separate event
+   specifications from word designators.  Static for now */
+static char *history_event_delimiter_chars = HISTORY_EVENT_DELIMITERS;
+
 static char *get_history_word_specifier PARAMS((char *, char *, int *));
 static int history_tokenize_word PARAMS((const char *, int));
 static char **history_tokenize_internal PARAMS((const char *, int, int *));
@@ -112,7 +118,6 @@ rl_linebuf_func_t *history_inhibit_expansion_function;
 
 /* The last string searched for by a !?string? search. */
 static char *search_string;
-
 /* The last string matched by a !?string? search. */
 static char *search_match;
 
@@ -225,6 +230,7 @@ get_history_event (string, caller_index, delimiting_quote)
 
 #endif /* HANDLE_MULTIBYTE */
       if ((!substring_okay && (whitespace (c) || c == ':' ||
+          (history_event_delimiter_chars && member (c, history_event_delimiter_chars)) ||
 	  (history_search_delimiter_chars && member (c, history_search_delimiter_chars)) ||
 	  string[i] == delimiting_quote)) ||
 	  string[i] == '\n' ||
@@ -245,7 +251,7 @@ get_history_event (string, caller_index, delimiting_quote)
 
 #define FAIL_SEARCH() \
   do { \
-    history_offset = history_length; free (temp) ; return (char *)NULL; \
+    history_offset = history_length; xfree (temp) ; return (char *)NULL; \
   } while (0)
 
   /* If there is no search string, try to use the previous search string,
@@ -254,7 +260,7 @@ get_history_event (string, caller_index, delimiting_quote)
     {
       if (search_string)
         {
-          free (temp);
+          xfree (temp);
           temp = savestring (search_string);
         }
       else
@@ -272,6 +278,8 @@ get_history_event (string, caller_index, delimiting_quote)
       if (local_index == 0 || substring_okay)
 	{
 	  entry = current_history ();
+	  if (entry == 0)
+	    FAIL_SEARCH ();
 	  history_offset = history_length;
 	
 	  /* If this was a substring search, then remember the
@@ -285,7 +293,7 @@ get_history_event (string, caller_index, delimiting_quote)
 	      search_match = history_find_word (entry->line, local_index);
 	    }
 	  else
-	    free (temp);
+	    xfree (temp);
 
 	  return (entry->line);
 	}
@@ -508,7 +516,7 @@ postproc_subst_rhs ()
 	}
     }
   new[j] = '\0';
-  free (subst_rhs);
+  xfree (subst_rhs);
   subst_rhs = new;
   subst_rhs_len = j;
 }
@@ -519,9 +527,9 @@ postproc_subst_rhs ()
    the returned string.  Returns the new index into string in
    *END_INDEX_PTR, and the expanded specifier in *RET_STRING. */
 static int
-history_expand_internal (string, start, end_index_ptr, ret_string, current_line)
+history_expand_internal (string, start, qc, end_index_ptr, ret_string, current_line)
      char *string;
-     int start, *end_index_ptr;
+     int start, qc, *end_index_ptr;
      char **ret_string;
      char *current_line;	/* for !# */
 {
@@ -557,35 +565,12 @@ history_expand_internal (string, start, end_index_ptr, ret_string, current_line)
       event = current_line;
     }
   else
-    {
-      int quoted_search_delimiter = 0;
-
-      /* If the character before this `!' is a double or single
-	 quote, then this expansion takes place inside of the
-	 quoted string.  If we have to search for some text ("!foo"),
-	 allow the delimiter to end the search string. */
-#if defined (HANDLE_MULTIBYTE)
-      if (MB_CUR_MAX > 1 && rl_byte_oriented == 0)
-	{
-	  int ch, l;
-	  l = _rl_find_prev_mbchar (string, i, MB_FIND_ANY);
-	  ch = string[l];
-	  /* XXX - original patch had i - 1 ???  If i == 0 it would fail. */
-	  if (i && (ch == '\'' || ch == '"'))
-	    quoted_search_delimiter = ch;
-	}
-      else
-#endif /* HANDLE_MULTIBYTE */	  
-	if (i && (string[i - 1] == '\'' || string[i - 1] == '"'))
-	  quoted_search_delimiter = string[i - 1];
-
-      event = get_history_event (string, &i, quoted_search_delimiter);
-    }
+    event = get_history_event (string, &i, qc);
 	  
   if (event == 0)
     {
       *ret_string = hist_error (string, start, i, EVENT_NOT_FOUND);
-      free (result);
+      xfree (result);
       return (-1);
     }
 
@@ -599,7 +584,7 @@ history_expand_internal (string, start, end_index_ptr, ret_string, current_line)
   if (word_spec == (char *)&error_pointer)
     {
       *ret_string = hist_error (string, starting_index, i, BAD_WORD_SPEC);
-      free (result);
+      xfree (result);
       return (-1);
     }
 
@@ -632,8 +617,8 @@ history_expand_internal (string, start, end_index_ptr, ret_string, current_line)
 	{
 	default:
 	  *ret_string = hist_error (string, i+1, i+2, BAD_MODIFIER);
-	  free (result);
-	  free (temp);
+	  xfree (result);
+	  xfree (temp);
 	  return -1;
 
 	case 'q':
@@ -658,7 +643,7 @@ history_expand_internal (string, start, end_index_ptr, ret_string, current_line)
 	    {
 	      tstr++;
 	      t = savestring (tstr);
-	      free (temp);
+	      xfree (temp);
 	      temp = t;
 	    }
 	  break;
@@ -683,7 +668,7 @@ history_expand_internal (string, start, end_index_ptr, ret_string, current_line)
 	  if (tstr)
 	    {
 	      t = savestring (tstr);
-	      free (temp);
+	      xfree (temp);
 	      temp = t;
 	    }
 	  break;
@@ -759,8 +744,8 @@ history_expand_internal (string, start, end_index_ptr, ret_string, current_line)
 	    if (subst_lhs_len == 0)
 	      {
 		*ret_string = hist_error (string, starting_index, i, NO_PREV_SUBST);
-		free (result);
-		free (temp);
+		xfree (result);
+		xfree (temp);
 		return -1;
 	      }
 
@@ -769,8 +754,8 @@ history_expand_internal (string, start, end_index_ptr, ret_string, current_line)
 	    if (subst_lhs_len > l_temp)
 	      {
 		*ret_string = hist_error (string, starting_index, i, SUBST_FAILED);
-		free (result);
-		free (temp);
+		xfree (result);
+		xfree (temp);
 		return (-1);
 	      }
 
@@ -811,7 +796,7 @@ history_expand_internal (string, start, end_index_ptr, ret_string, current_line)
 			     temp + si + subst_lhs_len,
 			     l_temp - (si + subst_lhs_len));
 		    new_event[len] = '\0';
-		    free (temp);
+		    xfree (temp);
 		    temp = new_event;
 
 		    failed = 0;
@@ -847,14 +832,14 @@ history_expand_internal (string, start, end_index_ptr, ret_string, current_line)
 	      continue;		/* don't want to increment i */
 
 	    *ret_string = hist_error (string, starting_index, i, SUBST_FAILED);
-	    free (result);
-	    free (temp);
+	    xfree (result);
+	    xfree (temp);
 	    return (-1);
 	  }
 	}
       i += 2;
     }
-  /* Done with modfiers. */
+  /* Done with modifiers. */
   /* Believe it or not, we have to back the pointer up by one. */
   --i;
 
@@ -869,7 +854,7 @@ history_expand_internal (string, start, end_index_ptr, ret_string, current_line)
       else
 	x = savestring (temp);
 
-      free (temp);
+      xfree (temp);
       temp = x;
     }
 
@@ -877,7 +862,7 @@ history_expand_internal (string, start, end_index_ptr, ret_string, current_line)
   if (n >= result_len)
     result = (char *)xrealloc (result, n + 2);
   strcpy (result, temp);
-  free (temp);
+  xfree (temp);
 
   *end_index_ptr = i;
   *ret_string = result;
@@ -894,7 +879,7 @@ history_expand_internal (string, start, end_index_ptr, ret_string, current_line)
    1) If expansions did take place
    2) If the `p' modifier was given and the caller should print the result
 
-  If an error ocurred in expansion, then OUTPUT contains a descriptive
+  If an error occurred in expansion, then OUTPUT contains a descriptive
   error message. */
 
 #define ADD_STRING(s) \
@@ -928,7 +913,7 @@ history_expand (hstring, output)
      char **output;
 {
   register int j;
-  int i, r, l, passc, cc, modified, eindex, only_printing, dquote, flag;
+  int i, r, l, passc, cc, modified, eindex, only_printing, dquote, squote, flag;
   char *string;
 
   /* The output string, and its length. */
@@ -991,7 +976,7 @@ history_expand (hstring, output)
 
       /* `!' followed by one of the characters in history_no_expand_chars
 	 is NOT an expansion. */
-      for (i = dquote = 0; string[i]; i++)
+      for (i = dquote = squote = 0; string[i]; i++)
 	{
 #if defined (HANDLE_MULTIBYTE)
 	  if (MB_CUR_MAX > 1 && rl_byte_oriented == 0)
@@ -1012,6 +997,7 @@ history_expand (hstring, output)
 	     history expansion performed on it.
 	     Skip the rest of the line and break out of the loop. */
 	  if (history_comment_char && string[i] == history_comment_char &&
+	      dquote == 0 &&
 	      (i == 0 || member (string[i - 1], history_word_delimiters)))
 	    {
 	      while (string[i])
@@ -1022,6 +1008,13 @@ history_expand (hstring, output)
 	    {
 	      if (cc == 0 || member (cc, history_no_expand_chars))
 		continue;
+	      /* DQUOTE won't be set unless history_quotes_inhibit_expansion
+		 is set.  The idea here is to treat double-quoted strings the
+		 same as the word outside double quotes; in effect making the
+		 double quote part of history_no_expand_chars when DQUOTE is
+		 set. */
+	      else if (dquote && cc == '"')
+		continue;
 	      /* If the calling application has set
 		 history_inhibit_expansion_function to a function that checks
 		 for special cases that should not be history expanded,
@@ -1064,16 +1057,16 @@ history_expand (hstring, output)
 	  
       if (string[i] != history_expansion_char)
 	{
-	  free (result);
+	  xfree (result);
 	  *output = savestring (string);
 	  return (0);
 	}
     }
 
   /* Extract and perform the substitution. */
-  for (passc = dquote = i = j = 0; i < l; i++)
+  for (passc = dquote = squote = i = j = 0; i < l; i++)
     {
-      int tchar = string[i];
+      int qc, tchar = string[i];
 
       if (passc)
 	{
@@ -1130,8 +1123,14 @@ history_expand (hstring, output)
 	case '\'':
 	  {
 	    /* If history_quotes_inhibit_expansion is set, single quotes
-	       inhibit history expansion. */
-	    if (dquote == 0 && history_quotes_inhibit_expansion)
+	       inhibit history expansion, otherwise they are treated like
+	       double quotes. */
+	    if (squote)
+	      {
+	        squote = 0;
+	        ADD_CHAR (tchar);
+	      }
+	    else if (dquote == 0 && history_quotes_inhibit_expansion)
 	      {
 		int quote, slen;
 
@@ -1144,7 +1143,12 @@ history_expand (hstring, output)
 		strncpy (temp, string + quote, slen);
 		temp[slen - 1] = '\0';
 		ADD_STRING (temp);
-		free (temp);
+		xfree (temp);
+	      }
+	    else if (dquote == 0 && squote == 0 && history_quotes_inhibit_expansion == 0)
+	      {
+	        squote = 1;
+	        ADD_CHAR (string[i]);
 	      }
 	    else
 	      ADD_CHAR (string[i]);
@@ -1152,12 +1156,13 @@ history_expand (hstring, output)
 	  }
 
 	case -2:		/* history_comment_char */
-	  if (i == 0 || member (string[i - 1], history_word_delimiters))
+	  if ((dquote == 0 || history_quotes_inhibit_expansion == 0) &&
+	      (i == 0 || member (string[i - 1], history_word_delimiters)))
 	    {
 	      temp = (char *)xmalloc (l - i + 1);
 	      strcpy (temp, string + i);
 	      ADD_STRING (temp);
-	      free (temp);
+	      xfree (temp);
 	      i = l;
 	    }
 	  else
@@ -1171,6 +1176,7 @@ history_expand (hstring, output)
 	     characters in history_no_expand_chars, then it is not a
 	     candidate for expansion of any kind. */
 	  if (cc == 0 || member (cc, history_no_expand_chars) ||
+			 (dquote && cc == '"') ||
 	  		 (history_inhibit_expansion_function && (*history_inhibit_expansion_function) (string, i)))
 	    {
 	      ADD_CHAR (string[i]);
@@ -1190,20 +1196,20 @@ history_expand (hstring, output)
 		  temp = (char *)xmalloc (1 + strlen (result));
 		  strcpy (temp, result);
 		  ADD_STRING (temp);
-		  free (temp);
+		  xfree (temp);
 		}
 	      i++;
 	      break;
 	    }
 #endif
-
-	  r = history_expand_internal (string, i, &eindex, &temp, result);
+	  qc = squote ? '\'' : (dquote ? '"' : 0);
+	  r = history_expand_internal (string, i, qc, &eindex, &temp, result);
 	  if (r < 0)
 	    {
 	      *output = temp;
-	      free (result);
+	      xfree (result);
 	      if (string != hstring)
-		free (string);
+		xfree (string);
 	      return -1;
 	    }
 	  else
@@ -1213,9 +1219,9 @@ history_expand (hstring, output)
 		  modified++;
 		  if (*temp)
 		    ADD_STRING (temp);
-		  free (temp);
+		  xfree (temp);
 		}
-	      only_printing = r == 1;
+	      only_printing += r == 1;
 	      i = eindex;
 	    }
 	  break;
@@ -1224,7 +1230,7 @@ history_expand (hstring, output)
 
   *output = result;
   if (string != hstring)
-    free (string);
+    xfree (string);
 
   if (only_printing)
     {
@@ -1405,8 +1411,8 @@ history_arg_extract (first, last, string)
     }
 
   for (i = 0; i < len; i++)
-    free (list[i]);
-  free (list);
+    xfree (list[i]);
+  xfree (list);
 
   return (result);
 }
@@ -1416,11 +1422,11 @@ history_tokenize_word (string, ind)
      const char *string;
      int ind;
 {
-  register int i;
-  int delimiter;
+  register int i, j;
+  int delimiter, nestdelim, delimopen;
 
   i = ind;
-  delimiter = 0;
+  delimiter = nestdelim = 0;
 
   if (member (string[i], "()\n"))
     {
@@ -1428,6 +1434,22 @@ history_tokenize_word (string, ind)
       return i;
     }
 
+  if (ISDIGIT (string[i]))
+    {
+      j = i;
+      while (string[j] && ISDIGIT (string[j]))
+	j++;
+      if (string[j] == 0)
+	return (j);
+      if (string[j] == '<' || string[j] == '>')
+	i = j;			/* digit sequence is a file descriptor */
+      else
+	{
+	  i = j;
+	  goto get_word;	/* digit sequence is part of a word */
+	}
+    }
+
   if (member (string[i], "<>;&|$"))
     {
       int peek = string[i + 1];
@@ -1441,14 +1463,30 @@ history_tokenize_word (string, ind)
 	  i += 2;
 	  return i;
 	}
-      else if ((peek == '&' && (string[i] == '>' || string[i] == '<')) ||
-		(peek == '>' && string[i] == '&') ||
-		(peek == '(' && (string[i] == '>' || string[i] == '<')) || /* ) */
-		(peek == '(' && string[i] == '$')) /* ) */
+      else if (peek == '&' && (string[i] == '>' || string[i] == '<'))
+	{
+	  j = i + 2;
+	  while (string[j] && ISDIGIT (string[j]))	/* file descriptor */
+	    j++;
+	  if (string[j] =='-')		/* <&[digits]-, >&[digits]- */
+	    j++;
+	  return j;
+	}
+      else if ((peek == '>' && string[i] == '&') || (peek == '|' && string[i] == '>'))
 	{
 	  i += 2;
 	  return i;
 	}
+      /* XXX - separated out for later -- bash-4.2 */
+      else if ((peek == '(' && (string[i] == '>' || string[i] == '<')) || /* ) */
+	       (peek == '(' && string[i] == '$')) /*)*/
+	{
+	  i += 2;
+	  delimopen = '(';
+	  delimiter = ')';
+	  nestdelim = 1;
+	  goto get_word;
+	}
 #if 0
       else if (peek == '\'' && string[i] == '$')
         {
@@ -1464,9 +1502,25 @@ history_tokenize_word (string, ind)
 	}
     }
 
+  /* same code also used for $(...)/<(...)/>(...) above */
+  if (member (string[i], "!@?+*"))
+    {
+      int peek = string[i + 1];
+
+      if (peek == '(')		/*)*/
+	{
+	  /* Shell extended globbing patterns */
+	  i += 2;
+	  delimopen = '(';
+	  delimiter = ')';	/* XXX - not perfect */
+	  nestdelim = 1;
+	}
+    }
+
+get_word:
   /* Get word from string + i; */
 
-  if (member (string[i], HISTORY_QUOTE_CHARACTERS))
+  if (delimiter == 0 && member (string[i], HISTORY_QUOTE_CHARACTERS))
     delimiter = string[i++];
 
   for (; string[i]; i++)
@@ -1484,16 +1538,31 @@ history_tokenize_word (string, ind)
 	  continue;
 	}
 
+      /* delimiter must be set and set to something other than a quote if
+	 nestdelim is set, so these tests are safe. */
+      if (nestdelim && string[i] == delimopen)
+	{
+	  nestdelim++;
+	  continue;
+	}
+      if (nestdelim && string[i] == delimiter)
+	{
+	  nestdelim--;
+	  if (nestdelim == 0)
+	    delimiter = 0;
+	  continue;
+	}
+      
       if (delimiter && string[i] == delimiter)
 	{
 	  delimiter = 0;
 	  continue;
 	}
 
-      if (!delimiter && (member (string[i], history_word_delimiters)))
+      if (delimiter == 0 && (member (string[i], history_word_delimiters)))
 	break;
 
-      if (!delimiter && member (string[i], HISTORY_QUOTE_CHARACTERS))
+      if (delimiter == 0 && member (string[i], HISTORY_QUOTE_CHARACTERS))
 	delimiter = string[i];
     }
 
@@ -1590,7 +1659,7 @@ freewords (words, start)
   register int i;
 
   for (i = start; words[i]; i++)
-    free (words[i]);
+    xfree (words[i]);
 }
 
 /* Find and return the word which contains the character at index IND
@@ -1614,8 +1683,8 @@ history_find_word (line, ind)
     }
   s = words[wind];
   for (i = 0; i < wind; i++)
-    free (words[i]);
+    xfree (words[i]);
   freewords (words, wind + 1);
-  free (words);
+  xfree (words);
   return s;
 }
